<h2>Reset your password</h2>
<p>We received a request to reset your password. Click the button below to choose a new password.</p>
<a href="{{ recovery_url }}" class="button">Reset Password</a>
<p>Or copy and paste this link into your browser:</p>
<p style="word-break: break-all; font-size: 14px; color: #666;">{{ recovery_url }}</p>
<div class="footer">
    <p>If you didn't request a password reset, you can safely ignore this email.</p>
    <p>This link will expire in 1 hour.</p>
</div>
