<h2>Sign in to {{ site_name }}</h2>
<p>Click the button below to sign in to your account. No password required!</p>
<a href="{{ magic_link_url }}" class="button">Sign In</a>
<p>Or copy and paste this link into your browser:</p>
<p style="word-break: break-all; font-size: 14px; color: #666;">{{ magic_link_url }}</p>
<div class="footer">
    <p>If you didn't request this email, you can safely ignore it.</p>
    <p>This link will expire in 1 hour.</p>
</div>
