<h2>Confirm your email address</h2>
<p>Thank you for signing up! Please click the button below to confirm your email address.</p>
<a href="{{ confirmation_url }}" class="button">Confirm Email</a>
<p>Or copy and paste this link into your browser:</p>
<p style="word-break: break-all; font-size: 14px; color: #666;">{{ confirmation_url }}</p>
<div class="footer">
    <p>If you didn't create an account with {{ site_name }}, you can safely ignore this email.</p>
</div>
